import React, { StatelessComponent, useEffect, useState } from 'react';
import Grid from '@material-ui/core/Grid';
import IconButton from '@material-ui/core/IconButton';
import Icon from '@material-ui/core/Icon';
import Typography from '@material-ui/core/Typography';
import ReactPaginate from 'rc-pagination';
import localeInfo from 'rc-pagination/lib/locale/en_US';

import SelectSupplierCard from '../SelectSupplierCard';
import Props from './SuppliersList.interface';
import RadioButon from '../input/radio-button';
import SelectedSuppliersList from '../SelectedSuppliersList';
import WarningMessage from '../warning-message';
import Input from '../../commons/input';
import SearchBar from '../input/search-bar';
import './style.css';

export const SuppliersList: StatelessComponent<Props> = props => {
  const {
    suppliers,
    selectedSuppliersIds,
    selectedSuppliers,
    suppliersLabels,
    classes,
    searchForSuppliersEffect,
    changePageOfSuppliersEffect,
    resultsPerPage,
    currentPage,
    numberOfPages,
    inputsModel,
    setSuppliersReferencedFilter,
    searchForSuppliers,
    suppliersPageTitle,
  } = props;
  const [isReferencedSuppliers, setReferencedSuppliers] = useState(true);

  useEffect(() => {
    window.scrollTo(0, 0);
  }, []);

  useEffect(() => {
    setSuppliersReferencedFilter(isReferencedSuppliers);
    searchForSuppliersEffect();
  }, [isReferencedSuppliers]);

  const handleChange = (current: number, pageSize: number) => {
    changePageOfSuppliersEffect({ currentPage: current, resultsPerPage }, isReferencedSuppliers);
  };

  const getAllBadSuppliers = () => {
    const filtredNotReferencedSupplier = selectedSuppliers.filter((supplier: any) => supplier.approvedByRESG === false);
    const filtredSuppliersWithRiskyKYS = selectedSuppliers.filter(
      (supplier: any) =>
        supplier.extStatus.toLowerCase() === 'medium_high' || supplier.extStatus.toLowerCase() === 'high',
    );
    const filtredSuppliersWithUnknownKYS = selectedSuppliers.filter(
      (supplier: any) =>
        supplier.extStatus.toLowerCase() === 'blocked' ||
        supplier.extStatus.toLowerCase() === 'not_eval' ||
        supplier.extStatus.toLowerCase() === 'in_progress' ||
        supplier.extStatus.toLowerCase() === 'not_allowed' ||
        supplier.extStatus.toLowerCase() === 'no_fin_data' ||
        supplier.extStatus.toLowerCase() === 'eval_obsolete' ||
        supplier.extStatus.toLowerCase() === 'unknown',
    );

    return {
      filtredNotReferencedSupplier,
      filtredSuppliersWithRiskyKYS,
      filtredSuppliersWithUnknownKYS,
    };
  };

  const badSuppliers = getAllBadSuppliers();

  return (
    <Grid item xs={12} className={classes.globalContainer}>
      <Grid item xs={12}>
        <Typography variant="h1" className={classes.pageTitle}>
          {suppliersPageTitle}
        </Typography>
      </Grid>
      <Grid container className={classes.headerContainer}>
        <Grid item xs={12}>
          <SearchBar
            searchTitle={suppliersLabels.searchTitle}
            searchClearButtonLabel={suppliersLabels.searchClearButtonLabel}
            searchPlaceHolder={suppliersLabels.searchPlaceHolder}
            setSearchTermAction={searchForSuppliers}
            searchAction={searchForSuppliersEffect}
          />
        </Grid>
        <Grid item xs={12}>
          <RadioButon
            keyLabel={suppliersLabels.showOnlyReferencedSuppliersLabel}
            value={isReferencedSuppliers}
            config={{ choices: ['yes', 'no'] }}
            handleChange={setReferencedSuppliers}
            validation={{}}
          />
        </Grid>
      </Grid>
      <Grid item xs={10}>
        {badSuppliers.filtredSuppliersWithUnknownKYS.length > 0 && (
          <WarningMessage
            helperText={{
              content: 'selectingSupplierWithUnknownKYS',
              icon: 'warning',
              fontSize: 'large',
              type: 'error',
            }}
          />
        )}
        {badSuppliers.filtredSuppliersWithRiskyKYS.length > 0 && (
          <WarningMessage
            helperText={{ content: 'selectingSupplierWithRiskyKYS', icon: 'warning', fontSize: 'large', type: 'error' }}
          />
        )}
        {badSuppliers.filtredNotReferencedSupplier.length > 0 && (
          <WarningMessage
            helperText={{
              content: 'selectingNotReferencedSupplier',
              icon: 'warning',
              fontSize: 'large',
              type: 'error',
            }}
          />
        )}
        <Input keyLabel="supplierComments" {...inputsModel['supplierComments']} />
      </Grid>
      <Grid container className={classes.listsContainer}>
        <Grid item xs={9} className={classes.listSuppliersContainer}>
          <div className={classes.titleContainer}>
            <h2 className={classes.title}>{suppliersLabels.title}</h2>
            <IconButton className={classes.button} aria-label="Delete" color="primary">
              <Icon>filter_list</Icon>
            </IconButton>
          </div>
          <Grid container>
            {suppliers.map((supplier: any) => (
              <SelectSupplierCard
                key={supplier.id}
                supplier={supplier}
                isCardSelected={selectedSuppliersIds.includes(supplier.id)}
              />
            ))}
          </Grid>
          <div className={classes.paginationContainer}>
            <ReactPaginate
              current={currentPage}
              hideOnSinglePage
              total={numberOfPages * resultsPerPage}
              defaultPageSize={resultsPerPage}
              locale={localeInfo}
              onChange={(current, pageSize) => {
                handleChange(current, pageSize);
              }}
            />
          </div>
        </Grid>
        <Grid item xs={3}>
          <SelectedSuppliersList />
        </Grid>
      </Grid>
    </Grid>
  );
};

export default SuppliersList;
