import React, { FunctionComponent } from 'react';
import { omit } from 'lodash';
import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';

import { Props } from './Input.container';
import { InputRootProps } from './inputRoot.interface';
import RichTextEditor from './rich-text-editor';
import Autocomplete from './autocomplete';
import DateTimePicker from './date-time-picker';
import TextField from './text-field-or-area';
import SelectField from './selectfield';
import RadioButtons from './radio-button';
import FileUpload from './file-upload';
import Gdpr from './gdpr';
import DownloadSingleFile from './download-single-file';
import WarningMessage from '../warning-message';
import Criteria from './criteria';

export const input: FunctionComponent<Props> = props => {
  const {
    toggleTouched,
    inputsModel,
    config,
    value,
    type,
    classes,
    inputLabels,
    inputErrorTexts,
    keyLabel,
    updateField,
    errorText,
    placeholder,
    inputPlaceholders,
    handleSearch,
    resetSuggestions,
    helperText,
    addTemporaryIntl,
    inputSelectionCriteria,
  } = props;

  const toggleTouchedHandler = (keyLabel: string, type: string) => {
    const element: InputRootProps = inputsModel[keyLabel];
    // If it is a blur and the element is not already touched we toggle
    if (type === 'blur' && !element.touched) {
      toggleTouched(keyLabel);
      // if it is a focus on an empty and touched element we toggle
    } else if (type === 'focus' && (element.value === '' || element.value === null) && element.touched) {
      toggleTouched(keyLabel);
    }
  };

  const handleChangeSubCriteria = (receivedValue: number, subCriteria: any) => {
    let newSubCriteria = subCriteria;
    newSubCriteria.value = receivedValue;
    let newValue = value;
    newValue.forEach((element: any) => {
      element.criteriaSelectionDetails.forEach((elementt: any) => {
        if (elementt.code === subCriteria.code) {
          elementt = newSubCriteria;
        }
      });
    });
    updateField(newValue, keyLabel);
  };

  const handleChangeCriteria = (receivedValue: number, criteria: any) => {
    value.forEach((element: any, index: number) => {
      if (element.code === criteria.code) {
        element.value = receivedValue > 100 || receivedValue < 0 ? 100 : receivedValue;
        if (value[index + 1]) {
          value[index + 1].value = 100 - element.value;
        } else {
          value[index - 1].value = 100 - element.value;
        }
      }
    });
    updateField(value, keyLabel);
  };

  const handleRemoveSubCriteria = (subCriteria: any, criteria: any) => {
    let newValue = value;
    const newCrit = criteria.criteriaSelectionDetails.filter((element: any) => element.code !== subCriteria.code);
    newValue.forEach((element: any) => {
      if (element.code === criteria.code) {
        element.criteriaSelectionDetails = newCrit;
      }
    });
    updateField(newValue, keyLabel);
  };

  const handleAddSubCriteria = (subCriteria: any, criteria: any) => {
    let newValue = value;
    let newSubCriteria = subCriteria;
    let max = 0;
    newValue.forEach((element: any) => {
      if (element.code === criteria.code) {
        element.criteriaSelectionDetails.forEach((element: any) => {
          if (max < element.orderNum) {
            max = element.orderNum;
          }
        });
        newSubCriteria.orderNum = max + 1;
        element.criteriaSelectionDetails.push(newSubCriteria);
      }
    });
    addTemporaryIntl(newSubCriteria);
    updateField(newValue, keyLabel);
  };

  let input = null;
  const managedValue: string = value !== null ? value : '';
  const translatedLabel = inputLabels[keyLabel];
  const translatedErrorText = errorText && inputErrorTexts[errorText];
  const translatedPlaceholder = placeholder && inputPlaceholders[placeholder];
  const passingProps = omit(props, [
    'updateField',
    'toggleTouched',
    'inputsModel',
    'value',
    'classes',
    'type',
    'keyLabel',
    'errorText',
    'placeholder',
    'handleSearch',
    'resetSuggestions',
    'inputErrorTexts',
    'inputLabels',
    'inputPlaceholders',
  ]);
  switch (type) {
    case 'inputTextOrTextarea':
      input = (
        <TextField
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          placeholder={translatedPlaceholder}
          handleChange={(value: string) => updateField(value, keyLabel)}
          handleToggle={(type: string) => toggleTouchedHandler(keyLabel, type)}
          {...passingProps}
        />
      );
      break;
    case 'selectfield':
      input = (
        <SelectField
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          handleToggle={(type: string) => toggleTouchedHandler(keyLabel, type)}
          inputKey={keyLabel}
          {...passingProps}
        />
      );
      break;
    case 'richTextEditor':
      input = (
        <RichTextEditor
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          handleToggle={(type: string) => toggleTouchedHandler(keyLabel, type)}
          {...passingProps}
        />
      );
      break;
    case 'autocomplete':
      input = (
        <Autocomplete
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          placeholder={translatedPlaceholder}
          handleChange={(value: string) => updateField(value, keyLabel)}
          handleToggle={(type: string) => toggleTouchedHandler(keyLabel, type)}
          handleSearch={(query: string) => handleSearch(query, keyLabel, props.apiToCall)}
          resetSuggestions={() => resetSuggestions(keyLabel)}
          {...passingProps}
        />
      );
      break;
    case 'fileUpload':
      input = (
        <FileUpload
          value={[]}
          handleChange={(value: any) => updateField(value, keyLabel)}
          destination={config.fileDestination}
          keyLabel={keyLabel}
        />
      );
      break;
    case 'datePicker':
      input = (
        <DateTimePicker
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          handleToggle={(type: string) => toggleTouchedHandler(keyLabel, type)}
          {...passingProps}
        />
      );
      break;
    case 'radioButtons':
      input = (
        <RadioButtons
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          {...passingProps}
        />
      );
      break;
    case 'gdpr':
      input = (
        <Gdpr
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          {...passingProps}
        />
      );
      break;
    case 'downloadSingleFile':
      input = (
        <DownloadSingleFile
          value={managedValue}
          keyLabel={translatedLabel}
          errorText={translatedErrorText}
          handleChange={(value: string) => updateField(value, keyLabel)}
          {...passingProps}
        />
      );
      break;
    case 'selectionCriterias':
      input = (
        <Grid item xs={12}>
          <WarningMessage helperText={{ content: 'selectionCriteriaSection', type: 'errorwar', icon: 'warning' }} />
          <Typography className={classes.helperMessage}>{inputSelectionCriteria.helpCriteriasMessages}</Typography>
          <Grid container>
            {value.map((criteria: any) => {
              return (
                <Grid key={criteria.code} item xs={12}>
                  <Criteria
                    criteria={criteria}
                    errorText={translatedErrorText}
                    handleChangeSubCriteria={handleChangeSubCriteria}
                    handleChangeCriteria={handleChangeCriteria}
                    handleRemoveSubCriteria={handleRemoveSubCriteria}
                    handleAddSubCriteria={handleAddSubCriteria}
                  />
                </Grid>
              );
            })}
          </Grid>
        </Grid>
      );
      break;
    default:
      input = <div>Not treated yet</div>;
  }

  // For the mngt rules we will add a show property in each input conf and it will render null if
  // show is set to false and the input inside a Grid when set to true
  return props.show === undefined || props.show === true ? (
    <>
      {helperText && (
        <Grid item xs={12}>
          <WarningMessage helperText={helperText} />
        </Grid>
      )}
      <Grid item xs={12} md={config.length === 50 ? 6 : 12} className={classes.input}>
        {input}
      </Grid>
    </>
  ) : null;
};

export default input;
