import React, { FunctionComponent, useEffect, useState } from 'react';
import Downshift, { ControllerStateAndHelpers } from 'downshift';
import { Paper, InputAdornment, IconButton } from '@material-ui/core';
import Suggestion from './suggestion';
import ClearIcon from '@material-ui/icons/Clear';
import { Props, Item } from './Autocomplete.interface';
import { isEqual, omit } from 'lodash';
import TextFieldOrArea from '../text-field-or-area';

export const autocomplete: FunctionComponent<Props> = props => {
  const {
    classes,
    suggestions,
    handleSearch,
    handleChange,
    handleToggle,
    resetSuggestions,
    touched,
    valid,
    errorText,
    keyLabel,
    placeholder,
    apiToCall,
    value,
    muiPropsCustomConf,
    disabled,
    handleValue,
  } = props;
  const [waiting, setWaiting]: any = useState(null);
  const [selected, setSelected]: any = useState(null);

  // We listen to the change of value
  useEffect(() => {
    // If we have a value and no selected or wrong one we go look for one
    value && (!selected || (selected && selected.backId !== value)) && getSelectedItem();

    // If we have no value but a selected we set it to null
    !value && selected && setSelected(null);
  }, [value]);

  const inputValueChangeHandler = (inputValue: string, stateAndHelpers: ControllerStateAndHelpers<any>) => {
    // Should clear the items and suggestions before looking for new one
    stateAndHelpers.clearItems();
    suggestions && suggestions.length > 0 && resetSuggestions();

    const trimValue = inputValue.trim();
    const selectedItem = stateAndHelpers.selectedItem;
    const isSelectedItem = selectedItem && trimValue === selectedItem.value;

    // We deselect the item if the user clear the field
    if ((trimValue === '' || trimValue.length === 1) && selectedItem) {
      handleChange('');
      setSelected(null);
      stateAndHelpers.clearSelection();
    }

    // We can had specific rules for each api we will call

    // People rules to prevent call to api until the arpege is complete
    let peopleRules: any = true;
    if (apiToCall === 'People') {
      const arpegeStartRegex = /^[ax][0-9]{1,5}$|^[ax][0-9]{7,}$/gi;
      peopleRules = !trimValue.match(arpegeStartRegex);
    }

    // We will do a search only if the rules are respected,
    // we have more than 3 characters, it's not the selected item
    // and the user stop typing for at least 500ms
    clearTimeout(waiting);
    if (trimValue.length >= 3 && !isSelectedItem && peopleRules) {
      setWaiting(
        setTimeout(() => {
          handleSearch(trimValue);
        }, 500),
      );
    }
  };
  const selectHandler = (item: Item) => {
    // If we select a different item we set it
    item && value !== item.backId && handleChange(item.backId);
    // If we clear we set an empty string
    !item && value && handleChange('');
  };

  const deselectHandler = (clearSelection: any) => {
    setSelected(null);
    clearSelection();
    handleChange('');
  };

  // We get the item for the api and set the state accordingly
  const getSelectedItem = async () => {
    try {
      const item = await handleValue(value, apiToCall);
      setSelected(item);
    } catch (res) {
      setSelected(null);
    }
  };
  const showClearIcon = (readOnly: boolean) => {
    return readOnly ? classes.showIcon : '';
  };

  return (
    <Downshift
      onInputValueChange={inputValueChangeHandler}
      itemToString={(item: Item) => (item ? item.value : '')}
      onSelect={selectHandler}
    >
      {({
        getInputProps,
        getItemProps,
        getMenuProps,
        highlightedIndex,
        isOpen,
        selectedItem,
        clearSelection,
        selectItem,
      }) => {
        // If the selected from state is different from the one from Downshift we set it
        !isEqual(selectedItem, selected) && selectItem(selected);

        const passingMuiPropsCustomConf = omit(muiPropsCustomConf, ['InputProps']);
        const passingInputProps = muiPropsCustomConf && muiPropsCustomConf.InputProps;
        const textfieldProps: any = {
          muiPropsCustomConf: {
            ...passingMuiPropsCustomConf,
            InputProps: {
              ...passingInputProps,
              ...getInputProps({
                fullWidth: !selectedItem && !disabled,
                endAdornment: selectedItem && !disabled && (
                  <InputAdornment position="end" className={props.readOnly ? showClearIcon(props.readOnly) : ''}>
                    <IconButton onClick={() => deselectHandler(clearSelection)}>
                      <ClearIcon fontSize="small" />
                    </IconButton>
                  </InputAdornment>
                ),
              }),
            },
          },
          placeholder,
          handleToggle,
          touched,
          valid,
          errorText,
          keyLabel,
          handleChange: () => {},
        };

        return (
          <div className={classes.container}>
            <TextFieldOrArea className={props.readOnly && classes.inputReadOnly} {...textfieldProps} />
            <div {...getMenuProps()}>
              {isOpen && suggestions && (
                <Paper className={classes.paper} square>
                  {suggestions.map((suggestion: Item, index: number) => {
                    const suggestionProps: any = {
                      suggestion,
                      index,
                      itemProps: getItemProps({ item: suggestion }),
                      highlightedIndex,
                      selectedItem,
                    };
                    return <Suggestion key={suggestion.value} {...suggestionProps} />;
                  })}
                </Paper>
              )}
            </div>
          </div>
        );
      }}
    </Downshift>
  );
};
export default autocomplete;
