import React, { useState, FunctionComponent, useEffect } from 'react';
import DatePicker, { registerLocale } from 'react-datepicker';
import './react-datepicker.css';
import fr from 'date-fns/locale/fr';
import enGB from 'date-fns/locale/en-GB';
import { format, isEqual, isValid } from 'date-fns';
import { Props } from './DateTimePicker.interface';
import TextFieldOrArea from '../text-field-or-area';
import { omit } from 'lodash';
import dotProp from 'dot-prop-immutable';

registerLocale('fr-FR', fr);
registerLocale('en-GB', enGB);

export const dateTimePicker: FunctionComponent<Props> = props => {
  const { value, dateConfig, locale } = props;
  const [selectedDate, setSelectedDate] = useState(initDate(value));
  const [dateFormat, setDateFormat]: any = useState(undefined);

  useEffect(() => {
    setSelectedDate(initDate(value));
  }, [value]);

  useEffect(() => {
    if (dateConfig && dateConfig.type === 'date') {
      setDateFormat('dd/MM/yyyy');
    }
    if (dateConfig && dateConfig.type === 'dateAndTime' && locale === 'fr') {
      setDateFormat('dd/MM/yyyy HH:mm');
    }
    if (dateConfig && dateConfig.type === 'dateAndTime' && locale === 'en') {
      setDateFormat('dd/MM/yyyy hh:mm a');
    }
  }, [locale]);

  const selectDateHandler = (date: Date) => {
    // If the date is different we....
    if (selectedDate === undefined || !isEqual(date, selectedDate)) {
      // set it in our useState
      setSelectedDate(date);
      // save the value in the right format
      const savedValue =
        dateConfig && dateConfig.type === 'date'
          ? format(date, 'yyyy-MM-dd')
          : dateConfig && dateConfig.type === 'dateAndTime'
          ? date.toISOString()
          : null;
      props.handleChange(savedValue);
    }
  };

  // We remove the handleChange, for now we can only select the date by clicking on the picker
  let textfieldProps = omit(props, ['dateConfig', 'locale', 'value', 'handleChange']);
  // We add the autoComplete="off" to remove the autofill that goes in front of the picker
  textfieldProps = dotProp.set(textfieldProps, 'muiPropsCustomConf.autoComplete', 'off');

  return (
    <DatePicker
      selected={selectedDate}
      onChange={selectDateHandler}
      readOnly={props.readOnly}
      dateFormat={dateFormat}
      placeholderText={dateFormat && dateFormat.toLowerCase()}
      showTimeSelect={dateConfig && dateConfig.type === 'dateAndTime'}
      timeFormat={locale === 'fr' ? 'HH:mm' : undefined}
      timeIntervals={60}
      timeCaption={locale === 'fr' ? 'Heure' : undefined}
      customInput={<TextFieldOrArea handleChange={() => {}} {...textfieldProps} />}
      locale={locale === 'fr' ? 'fr-FR' : locale === 'en' ? 'en-GB' : undefined}
      minDate={dateConfig && dateConfig.minDate ? initDate(dateConfig.minDate) : undefined}
      maxDate={dateConfig && dateConfig.maxDate ? initDate(dateConfig.maxDate) : undefined}
      showMonthDropdown
      showYearDropdown
      dropdownMode="select"
    />
  );
};

export const initDate = (value: string | Date) => {
  return isValid(new Date(value)) ? new Date(value) : undefined;
};

export default dateTimePicker;
