import React, { FunctionComponent, useState, useEffect } from 'react';
import { EditorState, convertToRaw, ContentState } from 'draft-js';
import { Editor } from 'react-draft-wysiwyg';
import './react-draft-wysiwyg.css';
import draftToHtml from 'draftjs-to-html';
import htmlToDraft from 'html-to-draftjs';
import { Props } from './RichTextEditor.container';
import { FormControl, FormHelperText, InputLabel } from '@material-ui/core';

export const richTextEditor: FunctionComponent<Props> = props => {
  const { value, valid, touched, keyLabel, errorText } = props;
  const { handleToggle, handleChange, classes, validation } = props;
  const [editorState, setEditorState] = useState(initEditorState(value));
  const [wrapperClasses, setWrapperClasses] = useState([classes.wrapper]);
  const [toolbarClasses, setToolbarClasses] = useState([classes.toolbar]);
  const [toolbarHidden, setToolbarHidden] = useState(true);

  useEffect(() => {
    const html: string = editorState ? draftToHtml(convertToRaw(editorState.getCurrentContent())).trim() : null;
    if (html !== value) {
      setEditorState(initEditorState(value));
    }
  }, [value]);

  // On change we set the editor state and send the html to the store if it is different
  const editorStateChangeHandler = (editorState: any) => {
    setEditorState(editorState);
    const html: string = draftToHtml(convertToRaw(editorState.getCurrentContent())).trim();
    // If the value is not an empty string but the html is empty we set the value to ''
    if (html === '<p></p>' && value !== '') {
      handleChange('');
    }
    // If we have an html and it is different of our value we set it
    if (html !== value && html !== '<p></p>') {
      handleChange(html);
    }
  };

  // We had the invalid class if the input is not valid and touched
  if (!valid && touched) {
    if (wrapperClasses.indexOf(classes.invalid) === -1) {
      setWrapperClasses([...wrapperClasses, classes.invalid]);
    }
    if (toolbarClasses.indexOf(classes.invalid) === -1) {
      setToolbarClasses([...toolbarClasses, classes.invalid]);
    }
  }

  // We remove the invalid class it exists and the input is valid
  if (valid || !touched) {
    if (wrapperClasses.indexOf(classes.invalid) !== -1) {
      setWrapperClasses(wrapperClasses.filter(e => e !== classes.invalid));
    }
    if (toolbarClasses.indexOf(classes.invalid) !== -1) {
      setToolbarClasses(toolbarClasses.filter(e => e !== classes.invalid));
    }
  }

  // On focus we ...
  const focusHandler = () => {
    // show the toolbar
    setToolbarHidden(false);
    // set the touched property with handleToggle
    handleToggle('focus');
    // Add focus class to wrapper and toolbar
    setWrapperClasses([...wrapperClasses, classes.focus]);
    setToolbarClasses([...toolbarClasses, classes.focus]);
  };

  // On blur we...
  const blurHandler = () => {
    // hide the toolbar
    setToolbarHidden(true);
    // set the touched property with handleToggle
    handleToggle('blur');
    // remove focus class to wrapper and toolbar
    setWrapperClasses(wrapperClasses.filter(e => e !== classes.focus));
    setToolbarClasses(toolbarClasses.filter(e => e !== classes.focus));
  };

  return (
    <FormControl error={!valid && touched} fullWidth className={classes.formControl}>
      <InputLabel shrink classes={{ shrink: classes.shrinkLabel }} required={validation && validation.required}>
        {keyLabel}
      </InputLabel>
      <Editor
        toolbarHidden={toolbarHidden}
        editorState={editorState}
        wrapperClassName={wrapperClasses.join(' ')}
        toolbarClassName={toolbarClasses.join(' ')}
        editorClassName={classes.editor}
        onEditorStateChange={editorStateChangeHandler}
        toolbar={toolbarConfig}
        onFocus={focusHandler}
        onBlur={blurHandler}
      />
      {!valid && touched && <FormHelperText variant="outlined">{errorText}</FormHelperText>}
    </FormControl>
  );
};

export const initEditorState = (value: string) => {
  let editorState;
  if (value !== null || value !== '') {
    const contentBlock = htmlToDraft(value);
    const contentState = ContentState.createFromBlockArray(contentBlock.contentBlocks);
    editorState = EditorState.createWithContent(contentState);
  }
  if (!value) {
    editorState = EditorState.createEmpty();
  }
  return editorState;
};

const toolbarConfig: any = {
  options: ['inline', 'colorPicker', 'fontSize', 'textAlign', 'list', 'remove'],
  inline: {
    options: ['bold', 'italic', 'underline'],
  },
  // fontFamily: {
  //   options: ['Source Sans Pro','Roboto','Arial'],
  // },
  list: {
    options: ['unordered', 'ordered'],
  },
  textAlign: {
    inDropdown: true,
  },
  colorPicker: {
    colors: [
      '#E60028',
      '#23558C',
      '#35BD5B',
      '#009BCD',
      '#F17100',
      '#B9292D',
      '#dd4e4e',
      '#ED5565',
      '#FC6E51',
      '#FFCE54',
      '#A0D468',
      '#48CFAD',
      '#4FC1E9',
      '#5D9CEC',
      '#AC92EC',
      '#EC87C0',
      '#F5F7FA',
      '#656D78',
      '#000000',
      '#FFFFFF',
    ],
  },
};

export default richTextEditor;
