import axios, { AxiosRequestConfig } from 'axios';
import merge from 'lodash/merge';
import urlJoin from 'url-join';

import { ApiRequesterManager, AuthenticationManager, AuthenticationToken } from './interfaces';

/**
 * @class ApiRequester
 * @implements {ApiRequesterManager}
 */

enum RequestType {
  Get = 'GET',
  Post = 'POST',
  Put = 'PUT',
  Delete = 'DELETE',
}

export function ApiRequester(basename: string = '/', authenticationMgr: AuthenticationManager): ApiRequesterManager {
  const axiosInstance = axios.create({
    // baseURL: basename,
    xsrfCookieName: 'XSRF-TOKEN',
    xsrfHeaderName: 'XSRF-TOKEN',
    withCredentials: true,
  });
  const authenticationMananger = authenticationMgr;
  const baseName = basename;

  const addAuthorizationHeader = async (options: AxiosRequestConfig): Promise<any> => {
    const authenticationToken: AuthenticationToken = await authenticationMananger.getAuthenticationToken();
    const optionsWithAuthorization: AxiosRequestConfig = merge(options, {
      headers: { authorization: `Bearer ${authenticationToken.accessToken}` },
    });
    return optionsWithAuthorization;
  };

  const handleRequest = async (
    requestType: RequestType,
    url: string,
    options: AxiosRequestConfig = {},
    data: any = {},
  ): Promise<any> => {
    const optionsWithAuthorization: AxiosRequestConfig = await addAuthorizationHeader(options);
    const apiUrl = urlJoin(baseName, url);
    try {
      switch (requestType) {
        case RequestType.Get:
          return await axiosInstance.get(apiUrl, optionsWithAuthorization);
        case RequestType.Post:
          return await axiosInstance.post(apiUrl, data, optionsWithAuthorization);
        case RequestType.Put:
          return await axiosInstance.put(apiUrl, data, optionsWithAuthorization);
        case RequestType.Delete:
          return await axiosInstance.delete(apiUrl, optionsWithAuthorization);
      }
    } catch (error) {
      // 401 Unauthorized & no response => disconnected from SAFE/GAIA/GAIA_MOCK
      if (error.response.status === 401) {
        // force reload to be redirected to SAFE/GAIA/GAIA_MOCK login page
        window.location.reload(true);
        return;
      } else {
        throw error;
      }
    }
  };

  const apiRequester = {
    get: async (url: string, options?: AxiosRequestConfig): Promise<any> => {
      return handleRequest(RequestType.Get, url, options);
    },
    put: async (url: string, data?: any, options?: AxiosRequestConfig): Promise<any> => {
      return handleRequest(RequestType.Put, url, options, data);
    },
    post: async (url: string, data?: any, options?: AxiosRequestConfig): Promise<any> => {
      return handleRequest(RequestType.Post, url, options, data);
    },
    delete: async (url: string, options?: AxiosRequestConfig): Promise<any> => {
      return handleRequest(RequestType.Delete, url, options);
    },
    upload: async (url: string, files: FileList, options?: AxiosRequestConfig): Promise<any> => {
      const formData = new FormData();
      // tslint:disable-next-line:prefer-for-of
      for (let index = 0; index < files.length; index++) {
        const file: File = files[index];
        formData.append(file.name, file.slice());
      }
      return handleRequest(RequestType.Post, url, { ...options, data: formData });
    },
  };
  return apiRequester;
}
