import { requester } from '../../helpers/api';
import * as effects from './prModels.effects';
import { actionTypes } from './prModels.actions';
import inputsModel from '../../template/inputsModel';
import { InputsModel } from './prModels.interfaces';

describe('prModels.effects', () => {
  describe('getInputsModelEffect', () => {
    it('should getInputsModelEffect', () => {
      const dispatch = jest.fn();
      effects.getInputsModelEffect()(dispatch);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        type: actionTypes.SET_INPUTS_MODEL,
        payload: { inputsModel },
      });
    });
  });
  describe('editInputsModelEffect', () => {
    it('should editInputsModelEffect', () => {
      const dispatch = jest.fn();
      const getState = jest.fn(() => ({
        prModels: {
          inputsModel: {
            prId: {
              value: null,
              valid: true,
              touched: false,
              errorText: null,
            },
            projectName: {
              value: null,
              valid: false,
              touched: false,
              errorText: null,
            },
          },
        },
        purchaseRequests: {
          activePR: {
            prId: 'PR-TEST',
            projectName: null,
            validationProcess: null,
          },
        },
      }));
      effects.editInputsModelEffect()(dispatch, getState);
      expect(dispatch).toHaveBeenCalledTimes(2);
      expect(getState).toHaveBeenCalledTimes(2);
    });
  });

  describe('updateInputEffect', () => {
    it('should updateInputEffect', () => {
      const dispatch = jest.fn();
      const getState = jest.fn(() => ({
        prModels: {
          inputsModel: {
            prId: {
              value: null,
              valid: false,
              touched: false,
              errorText: null,
              hasManagementRule: true,
            },
          },
        },
        purchaseRequests: {
          activePR: {
            prId: 'PR-TEST',
            projectName: null,
            validationProcess: null,
          },
        },
      }));
      effects.updateInputEffect('test', 'prId')(dispatch, getState);
      expect(getState).toHaveBeenCalledTimes(1);
      expect(dispatch).toHaveBeenCalledTimes(2);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        type: actionTypes.UPDATE_INPUT,
        payload: {
          inputsModel: {
            prId: {
              value: 'test',
              valid: true,
              touched: true,
              errorText: null,
              hasManagementRule: true,
            },
          },
          prIsValid: false,
          specificationIsValid: {
            invalidSpecificationInputs: [],
            prSpecificationIsValid: true,
          },
        },
      });
    });
  });

  describe('updateFieldFunction', () => {
    it('should updateFieldFunction', () => {
      const prModels = {
        inputsModel: {
          inputTest: {
            validation: {
              required: true,
            },
            value: null,
            valid: false,
            touched: false,
            errorText: null,
          },
        },
        prIsValid: false,
      };
      const activePR = {
        validationProcess: null,
      };
      const updatePrModelState = effects.updateFieldFunction(prModels, 'inputTest', 'test', activePR);
      expect(updatePrModelState).toEqual({
        inputsModel: {
          inputTest: {
            ...prModels.inputsModel.inputTest,
            value: 'test',
            valid: true,
            touched: true,
          },
        },
        prIsValid: false,
        specificationIsValid: {
          invalidSpecificationInputs: [],
          prSpecificationIsValid: true,
        },
      });
    });
  });

  describe('getPrIsValid', () => {
    it('should get the updated prIsValid property', () => {
      let inputsModel: InputsModel = {
        key1: {
          valid: true,
          validation: {},
        },
        key2: {
          valid: true,
          validation: {},
        },
      };
      const activePR: any = {
        validationProcess: {
          businessApprovals: ['test'],
          financialApprovals: ['test'],
        },
      };
      expect(effects.getPrIsValid(inputsModel, activePR)).toBeTruthy();
      inputsModel.key1.valid = false;
      expect(effects.getPrIsValid(inputsModel, activePR)).toBeFalsy();
    });
  });

  describe('getInputsValidity', () => {
    it('should return the inputs validity', () => {
      let inputsModel: InputsModel = {
        key1: {
          valid: true,
          validation: {},
        },
        key2: {
          valid: true,
          validation: {},
        },
      };
      expect(effects.getInputsValidity(inputsModel)).toBeTruthy();
      inputsModel.key1.valid = false;
      expect(effects.getInputsValidity(inputsModel)).toBeFalsy();
    });
  });

  describe('getValidationProcessValidity', () => {
    it('should return the inputs validity', () => {
      const validPR: any = {
        validationProcess: {
          businessApprovals: ['test'],
          financialApprovals: ['test'],
        },
      };
      expect(effects.getValidationProcessValidity(validPR)).toBeTruthy();
      const unvalidPR: any = {
        validationProcess: {
          businessApprovals: [],
          financialApprovals: ['test'],
        },
      };
      expect(effects.getValidationProcessValidity(unvalidPR)).toBeFalsy();
      const emptyPR: any = {
        validationProcess: null,
      };
      expect(effects.getValidationProcessValidity(emptyPR)).toBeFalsy();
    });
  });

  describe('getPrSpecificationIsValid', () => {
    it('should set prSpecificationIsValid to true when all specification validation required field are valid', () => {
      const inputsModel: InputsModel = {
        key1: {
          valid: true,
          validation: {
            integratedToSpecification: true,
          },
        },
        key2: {
          valid: false,
          validation: {},
        },
        key3: {
          valid: true,
          validation: {
            integratedToSpecification: true,
          },
        },
      };

      const ret = effects.getPrSpecificationIsValid(inputsModel);
      expect(ret).toEqual({ prSpecificationIsValid: true, invalidSpecificationInputs: [] });
    });

    it('should set prSpecificationIsValid to false with invalid field when at least one specification validation required field is invalid', () => {
      const inputsModel: InputsModel = {
        key1: {
          valid: false,
          validation: {
            integratedToSpecification: true,
          },
        },
        key2: {
          valid: false,
          validation: {},
        },
        key3: {
          valid: true,
          validation: {
            integratedToSpecification: true,
          },
        },
      };

      const ret = effects.getPrSpecificationIsValid(inputsModel);
      expect(ret).toEqual({ prSpecificationIsValid: false, invalidSpecificationInputs: ['key1'] });
    });
  });

  describe('toggleTouchedEffect', () => {
    it('should toggleTouchedEffect', () => {
      const dispatch = jest.fn();
      const getState = jest.fn(() => ({
        prModels: {
          inputsModel: {
            inputTest: {
              validation: {
                required: true,
              },
              value: '',
              valid: false,
              touched: false,
              errorText: null,
            },
          },
        },
      }));
      effects.toggleTouchedEffect('inputTest')(dispatch, getState);
      expect(getState).toHaveBeenCalledTimes(1);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        type: actionTypes.TOGGLE_TOUCHED,
        payload: {
          inputsModel: {
            inputTest: {
              validation: {
                required: true,
              },
              value: '',
              valid: false,
              touched: true,
              errorText: 'required',
            },
          },
        },
      });
    });
  });

  describe('resetSuggestionsEffect', () => {
    it('should resetSuggestionsEffect', () => {
      const dispatch = jest.fn();
      const getState = jest.fn(() => ({
        prModels: {
          inputsModel: {
            inputTest: {
              suggestions: ['test', 'test2'],
            },
          },
        },
      }));
      effects.resetSuggestionsEffect('inputTest')(dispatch, getState);
      expect(getState).toHaveBeenCalledTimes(1);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        type: actionTypes.RESET_SUGGESTIONS,
        payload: {
          inputsModel: {
            inputTest: {
              suggestions: [],
            },
          },
        },
      });
    });
  });
  describe('handleSearchEffect', () => {
    it('should handleSearchEffect', async () => {
      requester.get = jest.fn(() =>
        Promise.resolve({
          data: ['test'],
        }),
      );
      const dispatch = jest.fn();
      const getState = jest.fn(() => ({
        prModels: {
          inputsModel: {
            inputTest: {
              suggestions: [],
            },
          },
        },
      }));
      effects.handleSearchEffect('doe', 'inputTest', 'People')(dispatch, getState);
    });
  });
  it('should reset inputs model', () => {
    const dispatch = jest.fn();
    effects.resetInputsModelEffect()(dispatch);
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.RESET_INPUTS_MODEL,
      payload: { inputsModel: null, loader: true },
    });
  });
  it('should get inputsModel values', () => {
    const inputsModel = {
      inputKey1: {
        value: 'test',
        doNotSendToBack: true,
        validation: {},
      },
      inputKey2: {
        value: 'Test 2',
        validation: {},
      },
      inputKey3: {
        value: ['Test', '3'],
        validation: {},
      },
    };
    const values = effects.getPrValues(inputsModel);
    expect(values).toEqual({
      inputKey2: 'Test 2',
      inputKey3: ['Test', '3'],
    });
  });
  it('should set the back codes', () => {
    const dispatch = jest.fn();
    const data = [
      {
        lang: 'EN',
        code: 'test',
        choices: [
          {
            code: 'code1',
          },
          {
            code: 'code2',
          },
        ],
      },
    ];
    effects.setBackCodesEffect(data)(dispatch);
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.SET_BACK_CODES,
      payload: {
        backCodes: {
          test: ['code1', 'code2'],
        },
      },
    });
  });
});
