import inputsModel from '../../template/inputsModel';
import {
  setInputsModel,
  editInputsModel,
  updateInput,
  toggleTouched,
  updateSuggestions,
  resetSuggestions,
  resetInputsModel,
  setBackCodes,
} from './prModels.actions';
import { checkValidity } from '../../services/shared';
import dotProp from 'dot-prop-immutable';
import { isEqual } from 'lodash';
import { InputsModel } from './prModels.interfaces';
import { Item } from '../../components/commons/input/autocomplete/Autocomplete.interface';
import { managementRules } from '../../services/managementRules';
import { requester } from '../../helpers/api';

export const getInputsModelEffect = () => (dispatch: any) => {
  dispatch(setInputsModel({ inputsModel }));
};

export const editInputsModelEffect = () => (dispatch: any, getState: any) => {
  const activePR = getState().purchaseRequests.activePR;
  const prModels = getState().prModels;
  Object.keys(prModels.inputsModel).forEach((inputKey: string) => {
    const newValue = activePR[inputKey];
    const oldValue = prModels.inputsModel[inputKey].value;
    if (!isEqual(newValue, oldValue) && newValue !== undefined) {
      dispatch(updateInputEffect(newValue, inputKey));
    }
  });
  dispatch(editInputsModel({ loader: false }));
};

export const updateInputEffect = (newValue: any, inputKey: string) => (dispatch: any, getState: any) => {
  // First we update the store with the update of the field
  const state = getState();
  const { prModels } = state;
  const { activePR } = state.purchaseRequests;
  const updatedPrModelState = updateFieldFunction(prModels, inputKey, newValue, activePR);
  dispatch(updateInput(updatedPrModelState));
  // Then if the input has management rule we call the managementRules function that will deal with it
  const hasManagementRule = !!prModels.inputsModel[inputKey].hasManagementRule;
  hasManagementRule && dispatch(managementRules(newValue, inputKey));
};

export const updateFieldFunction = (prModels: any, key: string, newValue: any, activePR: any) => {
  const validity = checkValidity(newValue, prModels.inputsModel[key].validation);
  let updatedPrModels = dotProp.merge(prModels, ['inputsModel', key], {
    value: newValue,
    touched: true,
    valid: validity.valid,
    errorText: validity.errorText,
  });
  const prIsValid = getPrIsValid(updatedPrModels.inputsModel, activePR);
  const specificationIsValid = getPrSpecificationIsValid(updatedPrModels.inputsModel);
  return { ...updatedPrModels, prIsValid, specificationIsValid };
};

export const getPrIsValid = (inputsModel: InputsModel, activePR: any) => {
  const inputsValidity = getInputsValidity(inputsModel);
  const validationProcessValidity = getValidationProcessValidity(activePR);
  return inputsValidity && validationProcessValidity;
};

export const getInputsValidity = (inputsModel: InputsModel) => {
  let prIsValid = true;
  Object.keys(inputsModel).forEach((input: string) => {
    const valid = inputsModel[input].valid;
    // If valid is defined we update prIsValid
    typeof valid === 'boolean' && (prIsValid = valid && prIsValid);
  });
  return prIsValid;
};

export const getValidationProcessValidity = (activePR: any) => {
  const { validationProcess } = activePR;
  return validationProcess
    ? validationProcess.businessApprovals.length > 0 && validationProcess.financialApprovals.length > 0
    : false;
};

export const getPrSpecificationIsValid = (inputsModel: InputsModel) => {
  let prSpecificationIsValid: boolean = true;
  const invalidSpecificationInputs: any[] = [];
  Object.keys(inputsModel).forEach((input: string) => {
    let valid = true;

    let integratedToSpecification: boolean = false;
    if (
      typeof inputsModel[input].validation === 'object' &&
      typeof inputsModel[input].validation.integratedToSpecification !== 'undefined' &&
      inputsModel[input].validation.integratedToSpecification
    ) {
      integratedToSpecification = true;
      if (
        typeof inputsModel[input] !== 'undefined' &&
        typeof inputsModel[input].valid === 'boolean' &&
        inputsModel[input].valid
      ) {
        valid = true;
      } else {
        valid = false;
        invalidSpecificationInputs.push(input);
      }
      // If valid is defined we update prSpecificationIsValid
      prSpecificationIsValid = valid && prSpecificationIsValid && integratedToSpecification;
    } else {
      integratedToSpecification = false;
    }
  });
  return { prSpecificationIsValid, invalidSpecificationInputs };
};

export const toggleTouchedEffect = (inputKey: string) => (dispatch: any, getState: any) => {
  const inputsModel = getState().prModels.inputsModel;
  const input = inputsModel[inputKey];
  const newTouched = !input.touched;
  const validity = checkValidity(input.value, input.validation);
  const updatedInputsModel = dotProp.merge(inputsModel, inputKey, {
    touched: newTouched,
    valid: validity.valid,
    errorText: validity.errorText,
  });
  dispatch(toggleTouched({ inputsModel: updatedInputsModel }));
};

export const handleSearchEffect = (query: string, inputKey: string, apiToCall: string) => async (
  dispatch: any,
  getState: any,
) => {
  const state = getState();
  switch (apiToCall) {
    case 'People':
      try {
        const res: any = await requester.get(`people?q=${query}&limit=10`);
        const formatedSuggestions = res.data.map((person: any) => ({
          backId: person.igg,
          value: `${person.firstName} ${person.lastName} ${person.rtfeId}`,
        }));
        const updatedInputsModel = dotProp.merge(state.prModels.inputsModel, inputKey, {
          suggestions: formatedSuggestions,
        });
        dispatch(updateSuggestions({ inputsModel: updatedInputsModel }));
      } catch (res) {
        console.log(res);
      }
      break;
    case 'Sakkarah':
      try {
        const locale = state.intl.locale;
        const res: any = await requester.get(`organizational-units?q=${query}&limit=10&lang=${locale}`);
        const formatedSuggestions = res.data.map((entity: any) => ({
          backId: entity.id,
          value: locale === 'fr' ? entity.labelFr : entity.labelEn,
        }));
        const updatedInputsModel = dotProp.merge(state.prModels.inputsModel, inputKey, {
          suggestions: formatedSuggestions,
        });
        dispatch(updateSuggestions({ inputsModel: updatedInputsModel }));
      } catch (res) {
        console.log(res);
      }
      break;
    case 'SH':
      try {
        const res = await requester.get(`contracts?contractReferenceCode=${query}&limit=10`);
        const formatedSuggestions = res.data.map((contract: any) => ({
          backId: contract.referenceCode,
          value: `${contract.referenceCode} - ${contract.title}`,
        }));
        const updatedInputsModel = dotProp.merge(state.prModels.inputsModel, inputKey, {
          suggestions: formatedSuggestions,
        });
        dispatch(updateSuggestions({ inputsModel: updatedInputsModel }));
      } catch (res) {
        console.log(res);
      }
      break;
  }
};

export const resetSuggestionsEffect = (inputKey: string) => (dispatch: any, getState: any) => {
  const updatedInputsModel = dotProp.merge(getState().prModels.inputsModel, inputKey, {
    suggestions: [],
  });
  dispatch(resetSuggestions({ inputsModel: updatedInputsModel }));
};

export const handleAutoCompleteValueEffect = (value: string, apiToCall: string) => async (
  dispatch: any,
  getState: any,
) => {
  const state = getState();
  switch (apiToCall) {
    case 'People':
      try {
        const res: any = await requester.get(`people/${value}`);
        const item: Item = {
          backId: value,
          value: `${res.data.firstName} ${res.data.lastName} ${res.data.rtfeId}`,
        };
        return item;
      } catch (res) {
        console.log(res);
      }
      break;
    case 'Sakkarah':
      try {
        const res: any = await requester.get(`organizational-units/${value}`);
        const locale = state.intl.locale;
        const item: Item = {
          backId: value,
          value: locale === 'fr' ? res.data.labelFr : res.data.labelEn,
        };
        return item;
      } catch (res) {
        console.log(res);
      }
      break;
    case 'SH':
      try {
        const res: any = await requester.get(`contracts/${value}`);
        const item: Item = {
          backId: value,
          value: `${res.data.referenceCode} - ${res.data.title}`,
        };
        return item;
      } catch (res) {
        console.log(res);
      }
      break;
  }
};

export const resetInputsModelEffect = () => (dispatch: any) => {
  dispatch(resetInputsModel({ inputsModel: null, loader: true }));
};

export const getPrValues = (inputsModel: InputsModel) => {
  const prValues: { [key: string]: any } = {};
  Object.keys(inputsModel).forEach((inputKey: string) => {
    const value = inputsModel[inputKey].value;
    !inputsModel[inputKey].doNotSendToBack && (prValues[inputKey] = value);
  });
  return prValues;
};

export const setBackCodesEffect = (data: any) => (dispatch: any) => {
  interface Choice {
    code: string;
    label: string;
  }
  let newBackCodes: {
    [key: string]: string;
  } = {};
  data
    .filter((labels: any) => labels.lang === 'EN')
    .map((label: any) => {
      label.choices &&
        (newBackCodes = {
          ...newBackCodes,
          [label.code]: label.choices.map((choice: Choice) => choice.code),
        });
    });
  dispatch(setBackCodes({ backCodes: newBackCodes }));
};
