import * as effects from './suppliers.effects';
import { actionTypes } from './suppliers.actions';
import { requester } from '../../helpers/api';

describe('Suppliers effects', () => {
  let mockState: {
    [key: string]: any;
  };
  beforeEach(() => {
    mockState = {
      suppliers: {
        numberOfPages: 0,
        allSuppliers: [],
        selectedSuppliers: [],
        selectedSuppliersIds: [],
        paging: {
          currentPage: 1,
          resultsPerPage: 16,
        },
        searchTerm: '',
        isFiltredSuppliers: true,
      },
    };
  });

  describe('setSuppliers effect', () => {
    it('should set the userTasks', () => {
      const dispatch = jest.fn();
      const payload = [
        {
          supplierName: 'CAPGEMINI',
          supplierRiskLevel: 'low',
          isSupplierReferenced: false,
        },
        {
          supplierName: 'SOPRA',
          supplierRiskLevel: 'medium_low',
          isSupplierReferenced: true,
        },
      ];

      effects.setSuppliersEffect(payload, 2)(dispatch);
      expect(dispatch).toBeCalledTimes(1);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        payload: { allSuppliers: payload, numberOfPages: 2 },
        type: actionTypes.SUPPLIERS.SET_ALL,
      });
    });
  });

  describe('syncSuppliersInPRInputStoreEffect effect', () => {
    it('should change results page', () => {
      const dispatch = jest.fn();
      const mockedState = { suppliers: { selectedSuppliersIds: ['id1, id2'] } };
      const getState = jest.fn(() => mockedState);
      const payload = {
        paging: {
          currentPage: 1,
          resultsPerPage: 16,
        },
      };

      effects.syncSuppliersInPRInputStoreEffect()(dispatch, getState);
      expect(dispatch).toBeCalledTimes(1);
    });
  });

  describe('selectSupplierEffect effect', () => {
    it('should set supplier then sync data to input store', async () => {
      const dispatch = jest.fn();
      const mockedState = { suppliers: { selectedSuppliersIds: ['id1, id2'] } };
      const getState = jest.fn(() => mockedState);
      const payload = {
        paging: {
          currentPage: 1,
          resultsPerPage: 16,
        },
      };

      await effects.selectSupplierEffect('select payload')(dispatch, getState);
      expect(dispatch).toBeCalledTimes(2);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        payload: 'select payload',
        type: actionTypes.SUPPLIERS.SELECT,
      });
    });
  });

  describe('unselectSupplierEffect effect', () => {
    it('should unselect supplier then sync dat  a to input store', async () => {
      const dispatch = jest.fn();
      const mockedState = { suppliers: { selectedSuppliersIds: ['id1, id2'] } };
      const getState = jest.fn(() => mockedState);
      const payload = {
        paging: {
          currentPage: 1,
          resultsPerPage: 16,
        },
      };

      await effects.unselectSupplierEffect('unselect payload')(dispatch, getState);
      expect(dispatch).toBeCalledTimes(2);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        payload: 'unselect payload',
        type: actionTypes.SUPPLIERS.UNSELECT,
      });
    });
  });

  describe('changePageOfSuppliersEffect effect', () => {
    it('should change results page', () => {
      const dispatch = jest.fn();
      const payload = {
        paging: {
          currentPage: 1,
          resultsPerPage: 16,
        },
      };

      effects.changePageOfSuppliersEffect(payload)(dispatch);
      expect(dispatch).toBeCalledTimes(2);
      expect(dispatch).toHaveBeenNthCalledWith(1, {
        payload: { paging: payload.paging },
        type: actionTypes.SUPPLIERS.CHANGE_PAGE,
      });
    });
  });

  describe('getOneSupplierByIdEffect effect', () => {
    let getSuccessMockData: {};

    beforeEach(() => {
      getSuccessMockData = {
        supplier: {},
      };
    });

    it('should getOne supplier by Id', async () => {
      requester.get = jest.fn(() =>
        Promise.resolve({
          status: 200,
          data: getSuccessMockData,
        }),
      );
      const dispatch = jest.fn();

      await effects.getOneSupplierByIdEffect('supplierId')(dispatch);
      expect(dispatch).toBeCalledTimes(1);
    });
  });

  describe('getSelectedSuppliersListEffect effect', () => {
    let getSuccessMockData: {};

    beforeEach(() => {
      getSuccessMockData = {
        supplier: {},
      };
    });

    it('should getOne supplier for each Id in list', async () => {
      requester.get = jest.fn(() =>
        Promise.resolve({
          status: 200,
          data: getSuccessMockData,
        }),
      );
      const dispatch = jest.fn();
      const getState = jest.fn(() => mockState);
      const supplierListIds: any[] = [{}, {}, {}];
      await effects.getSelectedSuppliersListEffect(supplierListIds)(dispatch, getState);
      expect(dispatch).toBeCalledTimes(3);
    });
  });

  describe('searchForSuppliersEffect effect', () => {
    let getSuccessMockData: {};

    beforeEach(() => {
      getSuccessMockData = {
        suppliers: {
          numberOfPages: 0,
          allSuppliers: [],
          selectedSuppliers: [],
          selectedSuppliersIds: [],
          paging: {
            currentPage: 1,
            resultsPerPage: 16,
          },
          searchTerm: '',
          isFiltredSuppliers: true,
        },
      };
    });

    it('should search for suppliers', async () => {
      requester.get = jest.fn(() =>
        Promise.resolve({
          status: 200,
          data: getSuccessMockData,
        }),
      );
      const dispatch = jest.fn();
      const getState = jest.fn(() => mockState);

      await effects.searchForSuppliersEffect()(dispatch, getState);
      expect(getState).toBeCalledTimes(1);
    });
  });
});
