import {
  acceptedFilesFunc,
  rejectedFilesFunc,
  trimFileExtension,
  generateFileRows,
  deleteFileFunc,
  addFile,
} from './file-upload-utils';

describe('acceptedFilesFunc', () => {
  it('should return an array of file when mimetype is defined', () => {
    const mockedFiles: any[] = [
      {
        type: 'text/css',
      },
      {
        type: 'image/jpeg',
      },
    ];

    const acceptedFilesArray = acceptedFilesFunc(mockedFiles);

    expect(acceptedFilesArray.length).toEqual(mockedFiles.length);
    expect(acceptedFilesArray).toEqual(mockedFiles);
  });

  it('should remove files with invalid type from array', () => {
    const mockedFiles: any[] = [
      {
        type: 'text/css',
      },
      {
        type: 'text/javascript',
      },
      {
        type: 'image/jpeg',
      },
    ];

    const expectedFiles = [
      {
        type: 'text/css',
      },
      {
        type: 'image/jpeg',
      },
    ];

    const acceptedFilesArray = acceptedFilesFunc(mockedFiles);

    expect(acceptedFilesArray.length).toEqual(2);
    expect(acceptedFilesArray).toEqual(expectedFiles);
  });
});

describe('rejectedFilesFunc', () => {
  it('should return an empty array when mimetype is defined', () => {
    const mockedFiles: any[] = [
      {
        type: 'text/css',
      },
      {
        type: 'image/jpeg',
      },
    ];

    const acceptedFilesArray = rejectedFilesFunc(mockedFiles);

    expect(acceptedFilesArray.length).toEqual(0);
    expect(acceptedFilesArray).toEqual([]);
  });

  it('should add files with invalid type to array', () => {
    const mockedFiles = [
      {
        type: 'text/css',
      },
      {
        type: 'text/javascript',
      },
      {
        type: 'image/jpeg',
      },
    ];

    const expectedFiles = [
      {
        type: 'text/javascript',
      },
    ];

    const acceptedFilesArray = rejectedFilesFunc(mockedFiles);

    expect(acceptedFilesArray.length).toEqual(1);
    expect(acceptedFilesArray).toEqual(expectedFiles);
  });
});

describe('trimFileExtension', () => {
  it('should return fileName if fileName.ext is passed as argument', () => {
    expect(trimFileExtension('fileName.ext')).toEqual('fileName');
  });
  it('should return fileName.test if fileName.test.ext is passed as argument', () => {
    expect(trimFileExtension('fileName.test.ext')).toEqual('fileName.test');
  });
  it('should return fileName.16.05.2016 if fileName.16.05.2016.ext is passed as argument', () => {
    expect(trimFileExtension('fileName.16.05.2016.ext')).toEqual('fileName.16.05.2016');
  });
});

describe('generateFileRows', () => {
  it('should call generateFileTableRowFuncParam func with provided parameters', () => {
    const mockedFiles = [{ number: 1 }, { number: 2 }];
    const deleteFileParam = jest.fn();
    const setFilesParam = jest.fn();
    const filesDestination = 'internal';
    const generateFileTableRowFuncParam = jest.fn(() => 'Hello');

    const expectedReturn = ['Hello', 'Hello'];

    const tableBody = generateFileRows(
      mockedFiles,
      deleteFileParam,
      setFilesParam,
      filesDestination,
      generateFileTableRowFuncParam,
    );
    expect(generateFileTableRowFuncParam).toBeCalledTimes(2);
    expect(tableBody).toEqual(expectedReturn);
  });

  it('should call return an empty array when file array is empty', () => {
    const mockedFiles = 'not an array';
    const deleteFileParam = jest.fn();
    const setFilesParam = jest.fn();
    const filesDestination = 'internal';
    const generateFileTableRowFuncParam = jest.fn(() => 'Hello');

    const expectedReturn: any = [];

    const tableBody = generateFileRows(
      mockedFiles,
      deleteFileParam,
      setFilesParam,
      filesDestination,
      generateFileTableRowFuncParam,
    );
    expect(generateFileTableRowFuncParam).toBeCalledTimes(0);
    expect(tableBody).toEqual(expectedReturn);
  });

  it('should call return an empty array when file array is undefined', () => {
    const mockedFiles = undefined;
    const deleteFileParam = jest.fn();
    const setFilesParam = jest.fn();
    const filesDestination = 'internal';
    const generateFileTableRowFuncParam = jest.fn(() => 'Hello');

    const expectedReturn: any = [];

    const tableBody = generateFileRows(
      mockedFiles,
      deleteFileParam,
      setFilesParam,
      filesDestination,
      generateFileTableRowFuncParam,
    );
    expect(generateFileTableRowFuncParam).toBeCalledTimes(0);
    expect(tableBody).toEqual(expectedReturn);
  });
});

describe('deleteFileFunc', () => {
  it('should remove fileIdnex object and pass updated array to files state hook', () => {
    const mockedFiles: any[] = [
      {
        id: 1,
        type: 'text/css',
      },
      {
        id: 2,
        type: 'image/jpeg',
      },
      {
        id: 3,
        type: 'text/css',
      },
      {
        id: 4,
        type: 'text/css',
      },
      {
        id: 5,
        type: 'text/css',
      },
      {
        id: 6,
        type: 'image/jpeg',
      },
    ];
    const setFilesParam = jest.fn();
    const expectedReturn = [
      {
        id: 1,
        type: 'text/css',
      },
      {
        id: 2,
        type: 'image/jpeg',
      },
      {
        id: 3,
        type: 'text/css',
      },
      {
        id: 5,
        type: 'text/css',
      },
      {
        id: 6,
        type: 'image/jpeg',
      },
    ];

    const tableBody = deleteFileFunc(3, setFilesParam, mockedFiles);
    expect(setFilesParam).toBeCalledTimes(1);
    expect(setFilesParam).toBeCalledWith(expectedReturn);
  });
});

describe('addFile', () => {
  it('should call handleChangeParam and setFilesParam', () => {
    const mockedFiles: any[] = [
      {
        id: 1,
        type: 'text/css',
      },
      {
        id: 2,
        type: 'image/jpeg',
      },
      {
        id: 3,
        type: 'text/css',
      },
    ];
    const mockedAcceptedFilesParam: any[] = [
      {
        id: 'cd',
        type: 'text/css',
      },
      {
        id: 'ab',
        type: 'image/jpeg',
      },
    ];
    const handleChangeParam = jest.fn();
    const setFilesParam = jest.fn();

    addFile(mockedAcceptedFilesParam, setFilesParam, mockedFiles, handleChangeParam);
    expect(handleChangeParam).toBeCalledTimes(1);
    expect(setFilesParam).toBeCalledTimes(1);
    expect(handleChangeParam).toBeCalledWith(...mockedFiles, ...mockedAcceptedFilesParam);
    expect(setFilesParam).toBeCalledWith([...mockedFiles, ...mockedAcceptedFilesParam]);
  });
});
