import { requester } from '../helpers/api';
import { managementRules } from './managementRules';
import { actionTypes } from './../redux/prModels/prModels.actions';
import { gdprValue } from '../template/gdpr';

describe('Management rules handler', () => {
  let dispatch: any;
  beforeEach(() => {
    dispatch = jest.fn();
  });

  it('should calculated the daily rate', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          amount: {
            value: '5000',
          },
          estimatedNbMandays: {
            value: '10',
          },
          dailyRate: {
            value: null,
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules('1000', 'amount')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenCalled();
  });

  it('should show the gdpr treatment', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          gdprConsultingProviders: {
            touched: false,
            valid: true,
            value: [],
            show: false,
            validation: {},
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules(true, 'gdprEntrustedToProvider')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          gdprConsultingProviders: {
            touched: false,
            valid: false,
            value: [{ ...gdprValue, orderNum: 0 }],
            show: true,
            validation: {
              gdpr: true,
            },
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should hide the gdpr treatment', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          gdprConsultingProviders: {
            touched: false,
            valid: false,
            value: [{ ...gdprValue, orderNum: 0 }],
            show: true,
            validation: {
              gdpr: true,
            },
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules(false, 'gdprEntrustedToProvider')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          gdprConsultingProviders: {
            touched: false,
            valid: true,
            value: [],
            show: false,
            validation: {},
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should change projectEntityCode and contactId', async () => {
    requester.get = jest.fn(() =>
      Promise.resolve({
        data: {
          organizationUnitCode: '987654321',
        },
      }),
    );

    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          projectEntityCode: {
            value: '123456789',
          },
          contactId: {
            value: 'johnDoe',
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    await managementRules('janeDoe', 'beneficiaryUserId')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenCalled();
  });

  it('should show the contractId and extensionNumber fields', async () => {
    requester.put = jest.fn(() =>
      Promise.resolve({
        data: {
          projectStartDate: '2020-01-01',
        },
      }),
    );
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          contractId: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
          extensionNumber: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    await managementRules('EXTENSION', 'contractNatureCode')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenCalledTimes(2);
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          contractId: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
          extensionNumber: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should hide the contractId and extensionNumber fields', async () => {
    requester.put = jest.fn(() =>
      Promise.resolve({
        data: {
          projectStartDate: '2020-01-01',
        },
      }),
    );
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          contractId: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
          extensionNumber: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    await managementRules('INIT', 'contractNatureCode')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          contractId: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
          extensionNumber: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
          extensionComments: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should calculat the extensionNumber with the contracId', async () => {
    requester.get = jest.fn(() =>
      Promise.resolve({
        status: 200,
        data: {
          extensionNumber: 0,
        },
      }),
    );

    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          extensionNumber: {
            value: null,
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    await managementRules('c1234', 'contractId')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenCalled();
  });

  it('should show the extensionComments field', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          extensionComments: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules(4, 'extensionNumber')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          extensionComments: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should hide the extensionComments field', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          extensionComments: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules(null, 'extensionNumber')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          extensionComments: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should show the segmentTypeCode field', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          segmentTypeCode: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules('SUPPORT', 'segmentNatureCode')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          segmentTypeCode: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
        prIsValid: false,
      },
    });
  });

  it('should show the segmentTypeCode field', () => {
    const getState = jest.fn(() => ({
      prModels: {
        inputsModel: {
          segmentTypeCode: {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          },
        },
      },
      purchaseRequests: {
        activePR: {
          validationProcess: null,
        },
      },
    }));
    managementRules(null, 'segmentNatureCode')(dispatch, getState);
    expect(getState).toHaveBeenCalled();
    expect(dispatch).toHaveBeenNthCalledWith(1, {
      type: actionTypes.UPDATE_INPUT,
      payload: {
        inputsModel: {
          segmentTypeCode: {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        },
        prIsValid: false,
      },
    });
  });
  // it('should manage the minDate/maxDate for datePicker fields', () => {
  //   const getState = jest.fn(() => ({
  //     prModels: {
  //       inputsModel: {
  //         supplierOfferReceptionDateTime: {
  //           value: '2019-01-01',
  //         },
  //         supplierOralStartDate: {
  //           value: '2019-01-10',
  //         },
  //         supplierOralEndDate: {
  //           value: '2019-01-20',
  //         },
  //         estimatedDateSupplierSelection: {
  //           value: '2019-01-30',
  //         },
  //         estimatedContractSignatureDate: {
  //           value: '2019-02-01',
  //         },
  //         projectStartDate: {
  //           value: '2019-02-10',
  //         },
  //         projectEndDate: {
  //           value: '2019-12-31',
  //         },
  //       },
  //     },
  //     purchaseRequests: {
  //       activePR: {
  //         validationProcess: null,
  //       },
  //     },
  //   }));
  //   managementRules(null, 'projectEndDate')(dispatch, getState);
  //   expect(getState).toHaveBeenCalled();
  //   expect(dispatch).toHaveBeenNthCalledWith(1, {
  //     type: actionTypes.UPDATE_INPUT,
  //     payload: {
  //       inputsModel: {
  //         supplierOfferReceptionDateTime: {
  //           value: '2019-01-01',
  //           dateConfig: {
  //             maxDate: new Date('2019-01-09'),
  //           },
  //         },
  //         supplierOralStartDate: {
  //           value: '2019-01-10',
  //           dateConfig: {
  //             minDate: new Date('2019-01-02'),
  //             maxDate: new Date('2019-01-20'),
  //           },
  //         },
  //         supplierOralEndDate: {
  //           value: '2019-01-20',
  //           dateConfig: {
  //             minDate: new Date('2019-01-10'),
  //             maxDate: new Date('2019-01-30'),
  //           },
  //         },
  //         estimatedDateSupplierSelection: {
  //           value: '2019-01-30',
  //           dateConfig: {
  //             minDate: new Date('2019-01-20'),
  //             maxDate: new Date('2019-01-31'),
  //           },
  //         },
  //         estimatedContractSignatureDate: {
  //           value: '2019-02-01',
  //           dateConfig: {
  //             minDate: new Date('2019-01-31'),
  //             maxDate: new Date('2019-02-09'),
  //           },
  //         },
  //         projectStartDate: {
  //           value: '2019-02-10',
  //           dateConfig: {
  //             minDate: new Date('2019-02-02'),
  //             maxDate: new Date('2019-12-30'),
  //           },
  //         },
  //         projectEndDate: {
  //           value: '2019-12-31',
  //           dateConfig: {
  //             minDate: new Date('2019-02-11'),
  //           },
  //         },
  //       },
  //     },
  //   });
  // });
});
