import { updateInputEffect, updateInput } from '../redux/prModels';
import { gdprValue } from '../template/gdpr';
import dotProp from 'dot-prop-immutable';
import { getPrIsValid } from './../redux/prModels/prModels.effects';
import { isEqual } from 'lodash';
import { addDays, subDays } from 'date-fns';
import { requester } from '../helpers/api';

export const managementRules = (value: any, inputKey: string) => async (dispatch: any, getState: any) => {
  const state = getState();
  const activePR = state.purchaseRequests.activePR;
  const inputsModel = state.prModels.inputsModel;
  switch (inputKey) {
    case 'amount':
    case 'estimatedNbMandays':
      const amount = inputsModel['amount'].value;
      const estimatedNbMandays = inputsModel['estimatedNbMandays'].value;
      const oldDailyRate = inputsModel['dailyRate'].value;
      const newDailyRate = amount > 0 && estimatedNbMandays > 0 ? Math.round(amount / estimatedNbMandays) : null;
      newDailyRate !== oldDailyRate && dispatch(updateInputEffect(newDailyRate, 'dailyRate'));
      break;

    case 'gdprEntrustedToProvider':
      let updatedGdprInputsModel: any = { ...inputsModel };
      if (value) {
        updatedGdprInputsModel = dotProp.merge(updatedGdprInputsModel, 'gdprConsultingProviders', {
          touched: false,
          valid: false,
          value: [{ ...gdprValue, orderNum: 0 }],
          show: true,
          validation: {
            gdpr: true,
          },
        });
      } else {
        updatedGdprInputsModel = dotProp.merge(updatedGdprInputsModel, 'gdprConsultingProviders', {
          touched: false,
          valid: true,
          value: [],
          show: false,
          validation: {},
        });
      }
      !isEqual(inputsModel, updatedGdprInputsModel) &&
        dispatch(
          updateInput({
            inputsModel: updatedGdprInputsModel,
            prIsValid: getPrIsValid(updatedGdprInputsModel, activePR),
          }),
        );
      break;

    case 'beneficiaryUserId':
      // We manage the entity code rule
      const projectEntityCode = inputsModel['projectEntityCode'].value;
      const res = value && (await requester.get(`people/${value}`));
      const newProjectEntityCode = res && res.data.organizationUnitCode;
      projectEntityCode !== newProjectEntityCode &&
        dispatch(updateInputEffect(newProjectEntityCode, 'projectEntityCode'));

      // And we manage the contactId rule
      const contactId = inputsModel['contactId'].value;
      value && value !== contactId && dispatch(updateInputEffect(value, 'contactId'));
      break;

    case 'contractNatureCode':
      let updatedContractNatureCodeInputsModel: any = { ...inputsModel };
      const prId = activePR.prId;
      // First we manage the show/hide property for contractId, extensionNumber and extensionComments
      if (value === 'EXTENSION') {
        updatedContractNatureCodeInputsModel = dotProp.merge(updatedContractNatureCodeInputsModel, 'contractId', {
          value: null,
          valid: false,
          touched: false,
          show: true,
          validation: {
            required: true,
          },
        });
        updatedContractNatureCodeInputsModel = dotProp.merge(updatedContractNatureCodeInputsModel, 'extensionNumber', {
          value: null,
          valid: false,
          touched: false,
          show: true,
          validation: {
            required: true,
          },
        });
      } else {
        updatedContractNatureCodeInputsModel = dotProp.merge(updatedContractNatureCodeInputsModel, 'contractId', {
          value: null,
          valid: true,
          touched: false,
          show: false,
          validation: {},
        });
        updatedContractNatureCodeInputsModel = dotProp.merge(updatedContractNatureCodeInputsModel, 'extensionNumber', {
          value: null,
          valid: true,
          touched: false,
          show: false,
          validation: {},
        });
        updatedContractNatureCodeInputsModel = dotProp.merge(
          updatedContractNatureCodeInputsModel,
          'extensionComments',
          {
            value: null,
            valid: true,
            touched: false,
            show: false,
            validation: {},
          },
        );
      }
      await dispatch(
        updateInput({
          inputsModel: updatedContractNatureCodeInputsModel,
          prIsValid: getPrIsValid(updatedContractNatureCodeInputsModel, activePR),
        }),
      );
      // Then we get the new planning from back and set the new dates
      const resPlanning = value && (await requester.put(`purchases/${prId}/create-planning?requestType=${value}`));
      const newPlanning = resPlanning && resPlanning.data;
      newPlanning &&
        Object.keys(newPlanning).map(dateKey => {
          dispatch(updateInputEffect(newPlanning[dateKey], dateKey));
        });
      break;

    case 'contractId':
      const extensionNumber = inputsModel['extensionNumber'].value;
      const contractIdRes = value && (await requester.get(`contracts/${value}`));
      const newExtensionNumber = contractIdRes ? (contractIdRes.data.extensionNumber + 1).toString() : null;
      extensionNumber !== newExtensionNumber && dispatch(updateInputEffect(newExtensionNumber, 'extensionNumber'));
      break;

    case 'extensionNumber':
      let updatedExtensionNumberInputsModel: any = { ...inputsModel };
      const extensionCommentsShow = inputsModel['extensionComments'].show;
      value > 2 &&
        !extensionCommentsShow &&
        (updatedExtensionNumberInputsModel = dotProp.merge(updatedExtensionNumberInputsModel, 'extensionComments', {
          value: null,
          valid: false,
          touched: false,
          show: true,
          validation: {
            required: true,
          },
        }));
      value <= 2 &&
        extensionCommentsShow &&
        (updatedExtensionNumberInputsModel = dotProp.merge(updatedExtensionNumberInputsModel, 'extensionComments', {
          value: null,
          valid: true,
          touched: false,
          show: false,
          validation: {},
        }));
      !isEqual(inputsModel, updatedExtensionNumberInputsModel) &&
        dispatch(
          updateInput({
            inputsModel: updatedExtensionNumberInputsModel,
            prIsValid: getPrIsValid(updatedExtensionNumberInputsModel, activePR),
          }),
        );
      break;

    case 'suppliersIds':
      let updatedSuppliersIdsInputsModel: any = { ...inputsModel };
      const supplierCommentsShow = inputsModel['supplierComments'].show;
      const contractNatureCode = inputsModel['contractNatureCode'];
      const extensionNumberr = inputsModel['extensionNumber'];
      if (contractNatureCode.value && extensionNumberr.value) {
        value.length > 0 &&
          value.length < 3 &&
          !supplierCommentsShow &&
          (contractNatureCode.value === 'INIT' ||
            (contractNatureCode.value === 'EXTENSION' && extensionNumberr.value > 2 && extensionNumberr.value > 0)) &&
          (updatedSuppliersIdsInputsModel = dotProp.merge(updatedSuppliersIdsInputsModel, 'supplierComments', {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          }));
      } else {
        value.length > 0 &&
          value.length < 3 &&
          !supplierCommentsShow &&
          (updatedSuppliersIdsInputsModel = dotProp.merge(updatedSuppliersIdsInputsModel, 'supplierComments', {
            value: null,
            valid: false,
            touched: false,
            show: true,
            validation: {
              required: true,
            },
          }));
      }
      (value.length < 1 || value.length > 2) &&
        supplierCommentsShow &&
        (updatedSuppliersIdsInputsModel = dotProp.merge(updatedSuppliersIdsInputsModel, 'supplierComments', {
          value: null,
          valid: true,
          touched: false,
          show: false,
          validation: {},
        }));

      !isEqual(inputsModel, updatedSuppliersIdsInputsModel) &&
        dispatch(
          updateInput({
            inputsModel: updatedSuppliersIdsInputsModel,
            prIsValid: getPrIsValid(updatedSuppliersIdsInputsModel, activePR),
          }),
        );
      break;

    case 'segmentNatureCode':
      let updatedSegmentNatureInputsModel: any = { ...inputsModel };
      const segmentTypeCodeShow = inputsModel['segmentTypeCode'].show;
      value === 'SUPPORT' &&
        !segmentTypeCodeShow &&
        (updatedSegmentNatureInputsModel = dotProp.merge(updatedSegmentNatureInputsModel, 'segmentTypeCode', {
          value: null,
          valid: false,
          touched: false,
          show: true,
          validation: {
            required: true,
          },
        }));
      value !== 'SUPPORT' &&
        segmentTypeCodeShow &&
        (updatedSegmentNatureInputsModel = dotProp.merge(updatedSegmentNatureInputsModel, 'segmentTypeCode', {
          value: null,
          valid: true,
          touched: false,
          show: false,
          validation: {},
        }));
      !isEqual(inputsModel, updatedSegmentNatureInputsModel) &&
        dispatch(
          updateInput({
            inputsModel: updatedSegmentNatureInputsModel,
            prIsValid: getPrIsValid(updatedSegmentNatureInputsModel, activePR),
          }),
        );
      break;

    // case 'supplierOfferReceptionDateTime':
    // case 'supplierOralStartDate':
    // case 'supplierOralEndDate':
    // case 'estimatedDateSupplierSelection':
    // case 'estimatedContractSignatureDate':
    // case 'projectStartDate':
    // case 'projectEndDate':
    //   let updatedPlanningInputsModel = { ...inputsModel };
    //   const planning = {
    //     supplierOfferReceptionDateTime: inputsModel.supplierOfferReceptionDateTime.value,
    //     supplierOralStartDate: inputsModel.supplierOralStartDate.value,
    //     supplierOralEndDate: inputsModel.supplierOralEndDate.value,
    //     estimatedDateSupplierSelection: inputsModel.estimatedDateSupplierSelection.value,
    //     estimatedContractSignatureDate: inputsModel.estimatedContractSignatureDate.value,
    //     projectStartDate: inputsModel.projectStartDate.value,
    //     projectEndDate: inputsModel.projectEndDate.value,
    //   };
    //   updatedPlanningInputsModel = dotProp.merge(
    //     updatedPlanningInputsModel,
    //     'supplierOfferReceptionDateTime.dateConfig',
    //     {
    //       maxDate: subDays(new Date(planning.supplierOralStartDate), 1),
    //     },
    //   );
    //   updatedPlanningInputsModel = dotProp.merge(updatedPlanningInputsModel, 'supplierOralStartDate.dateConfig', {
    //     minDate: addDays(new Date(planning.supplierOfferReceptionDateTime), 1),
    //     maxDate: new Date(planning.supplierOralEndDate),
    //   });
    //   updatedPlanningInputsModel = dotProp.merge(updatedPlanningInputsModel, 'supplierOralEndDate.dateConfig', {
    //     minDate: new Date(planning.supplierOralStartDate),
    //     maxDate: new Date(planning.estimatedDateSupplierSelection),
    //   });
    //   updatedPlanningInputsModel = dotProp.merge(
    //     updatedPlanningInputsModel,
    //     'estimatedDateSupplierSelection.dateConfig',
    //     {
    //       minDate: new Date(planning.supplierOralEndDate),
    //       maxDate: subDays(new Date(planning.estimatedContractSignatureDate), 1),
    //     },
    //   );
    //   updatedPlanningInputsModel = dotProp.merge(
    //     updatedPlanningInputsModel,
    //     'estimatedContractSignatureDate.dateConfig',
    //     {
    //       minDate: addDays(new Date(planning.estimatedDateSupplierSelection), 1),
    //       maxDate: subDays(new Date(planning.projectStartDate), 1),
    //     },
    //   );
    //   updatedPlanningInputsModel = dotProp.merge(updatedPlanningInputsModel, 'projectStartDate.dateConfig', {
    //     minDate: addDays(new Date(planning.estimatedContractSignatureDate), 1),
    //     maxDate: planning.projectEndDate && subDays(new Date(planning.projectEndDate), 1),
    //   });
    //   updatedPlanningInputsModel = dotProp.merge(updatedPlanningInputsModel, 'projectEndDate.dateConfig', {
    //     minDate: addDays(new Date(planning.projectStartDate), 1),
    //   });
    //   !isEqual(inputsModel, updatedPlanningInputsModel) &&
    //     dispatch(updateInput({ inputsModel: updatedPlanningInputsModel }));
    //   break;
  }
};
